
import cffi
import sys


if sys.platform == 'win32':
    PLATFORM_TYPES = """
typedef long time_t;
typedef long suseconds_t;
typedef short h_addrtype_t;
typedef short h_length_t;
typedef short sa_family_t;
typedef unsigned short in_port_t;
"""
else:
    PLATFORM_TYPES = """
typedef long... time_t;
typedef long... suseconds_t;
typedef int h_addrtype_t;
typedef int h_length_t;
typedef int... sa_family_t;
typedef uint16_t in_port_t;
"""

TYPES = """
struct in_addr {
    uint32_t s_addr;
};

struct in6_addr {
    uint8_t s6_addr[16];
    ...;
};

struct timeval {
    time_t      tv_sec;
    suseconds_t tv_usec;
};

struct hostent {
   char         *h_name;
   char         **h_aliases;
   h_addrtype_t h_addrtype;
   h_length_t   h_length;
   char         **h_addr_list;
};

struct sockaddr {
    sa_family_t sa_family;
    ...;
};

struct sockaddr_in {
    sa_family_t       sin_family;
    in_port_t         sin_port;
    struct in_addr    sin_addr;
    ...;
};

struct sockaddr_in6 {
    sa_family_t         sin6_family;
    in_port_t           sin6_port;
    uint32_t            sin6_flowinfo;
    struct in6_addr     sin6_addr;
    uint32_t            sin6_scope_id;
    ...;
};

#define INET_ADDRSTRLEN ...
#define INET6_ADDRSTRLEN ...

/* DNS record types */
typedef enum {
  ARES_REC_TYPE_A = 1,
  ARES_REC_TYPE_NS = 2,
  ARES_REC_TYPE_CNAME = 5,
  ARES_REC_TYPE_SOA = 6,
  ARES_REC_TYPE_PTR = 12,
  ARES_REC_TYPE_MX = 15,
  ARES_REC_TYPE_TXT = 16,
  ARES_REC_TYPE_AAAA = 28,
  ARES_REC_TYPE_SRV = 33,
  ARES_REC_TYPE_NAPTR = 35,
  ARES_REC_TYPE_TLSA = 52,
  ARES_REC_TYPE_HTTPS = 65,
  ARES_REC_TYPE_CAA = 257,
  ARES_REC_TYPE_URI = 256,
  ARES_REC_TYPE_ANY = 255,
  ...
} ares_dns_rec_type_t;

/* DNS classes */
typedef enum {
  ARES_CLASS_IN = 1,
  ARES_CLASS_CHAOS = 3,
  ARES_CLASS_HESOID = 4,
  ARES_CLASS_NONE = 254,
  ARES_CLASS_ANY = 255,
  ...
} ares_dns_class_t;

/* DNS sections */
typedef enum {
  ARES_SECTION_ANSWER = 1,
  ARES_SECTION_AUTHORITY = 2,
  ARES_SECTION_ADDITIONAL = 3,
  ...
} ares_dns_section_t;

/* DNS Header opcodes */
typedef enum {
  ARES_OPCODE_QUERY = 0,
  ARES_OPCODE_IQUERY = 1,
  ARES_OPCODE_STATUS = 2,
  ARES_OPCODE_NOTIFY = 4,
  ARES_OPCODE_UPDATE = 5,
  ...
} ares_dns_opcode_t;

/* DNS Response codes */
typedef enum {
  ARES_RCODE_NOERROR = 0,
  ARES_RCODE_FORMERR = 1,
  ARES_RCODE_SERVFAIL = 2,
  ARES_RCODE_NXDOMAIN = 3,
  ARES_RCODE_NOTIMP = 4,
  ARES_RCODE_REFUSED = 5,
  ARES_RCODE_YXDOMAIN = 6,
  ARES_RCODE_YXRRSET = 7,
  ARES_RCODE_NXRRSET = 8,
  ARES_RCODE_NOTAUTH = 9,
  ARES_RCODE_NOTZONE = 10,
  ARES_RCODE_DSOTYPEI = 11,
  ARES_RCODE_BADSIG = 16,
  ARES_RCODE_BADKEY = 17,
  ARES_RCODE_BADTIME = 18,
  ARES_RCODE_BADMODE = 19,
  ARES_RCODE_BADNAME = 20,
  ARES_RCODE_BADALG = 21,
  ARES_RCODE_BADTRUNC = 22,
  ARES_RCODE_BADCOOKIE = 23,
  ...
} ares_dns_rcode_t;

/* DNS Header flags */
typedef enum {
  ARES_FLAG_QR = 1 << 0,
  ARES_FLAG_AA = 1 << 1,
  ARES_FLAG_TC = 1 << 2,
  ARES_FLAG_RD = 1 << 3,
  ARES_FLAG_RA = 1 << 4,
  ARES_FLAG_AD = 1 << 5,
  ARES_FLAG_CD = 1 << 6,
  ...
} ares_dns_flags_t;

/* DNS RR keys for accessing record fields */
typedef enum {
  /* A record */
  ARES_RR_A_ADDR = 1,

  /* AAAA record */
  ARES_RR_AAAA_ADDR = 1,

  /* NS record */
  ARES_RR_NS_NSDNAME = 1,

  /* CNAME record */
  ARES_RR_CNAME_CNAME = 1,

  /* SOA record */
  ARES_RR_SOA_MNAME = 1,
  ARES_RR_SOA_RNAME = 2,
  ARES_RR_SOA_SERIAL = 3,
  ARES_RR_SOA_REFRESH = 4,
  ARES_RR_SOA_RETRY = 5,
  ARES_RR_SOA_EXPIRE = 6,
  ARES_RR_SOA_MINIMUM = 7,

  /* PTR record */
  ARES_RR_PTR_DNAME = 1,

  /* MX record */
  ARES_RR_MX_PREFERENCE = 1,
  ARES_RR_MX_EXCHANGE = 2,

  /* TXT record */
  ARES_RR_TXT_DATA = 1,

  /* SRV record */
  ARES_RR_SRV_PRIORITY = 1,
  ARES_RR_SRV_WEIGHT = 2,
  ARES_RR_SRV_PORT = 3,
  ARES_RR_SRV_TARGET = 4,

  /* NAPTR record */
  ARES_RR_NAPTR_ORDER = 1,
  ARES_RR_NAPTR_PREFERENCE = 2,
  ARES_RR_NAPTR_FLAGS = 3,
  ARES_RR_NAPTR_SERVICES = 4,
  ARES_RR_NAPTR_REGEXP = 5,
  ARES_RR_NAPTR_REPLACEMENT = 6,

  /* CAA record */
  ARES_RR_CAA_CRITICAL = 1,
  ARES_RR_CAA_TAG = 2,
  ARES_RR_CAA_VALUE = 3,

  /* TLSA record */
  ARES_RR_TLSA_CERT_USAGE = 5201,
  ARES_RR_TLSA_SELECTOR = 5202,
  ARES_RR_TLSA_MATCH = 5203,
  ARES_RR_TLSA_DATA = 5204,

  /* HTTPS record */
  ARES_RR_HTTPS_PRIORITY = 6501,
  ARES_RR_HTTPS_TARGET = 6502,
  ARES_RR_HTTPS_PARAMS = 6503,

  /* URI record */
  ARES_RR_URI_PRIORITY = 25601,
  ARES_RR_URI_WEIGHT = 25602,
  ARES_RR_URI_TARGET = 25603,
  ...
} ares_dns_rr_key_t;

/* Opaque DNS record structures */
typedef struct ares_dns_record ares_dns_record_t;
typedef struct ares_dns_rr ares_dns_rr_t;

typedef int... ares_socket_t;
typedef int... ares_socklen_t;

#define ARES_FLAG_USEVC         ...
#define ARES_FLAG_PRIMARY       ...
#define ARES_FLAG_IGNTC         ...
#define ARES_FLAG_NORECURSE     ...
#define ARES_FLAG_STAYOPEN      ...
#define ARES_FLAG_NOSEARCH      ...
#define ARES_FLAG_NOALIASES     ...
#define ARES_FLAG_NOCHECKRESP   ...
#define ARES_FLAG_EDNS          ...
#define ARES_FLAG_NO_DFLT_SVR   ...

#define ARES_OPT_FLAGS          ...
#define ARES_OPT_TIMEOUT        ...
#define ARES_OPT_TRIES          ...
#define ARES_OPT_NDOTS          ...
#define ARES_OPT_UDP_PORT       ...
#define ARES_OPT_TCP_PORT       ...
#define ARES_OPT_SERVERS        ...
#define ARES_OPT_DOMAINS        ...
#define ARES_OPT_LOOKUPS        ...
#define ARES_OPT_SOCK_STATE_CB  ...
#define ARES_OPT_SORTLIST       ...
#define ARES_OPT_SOCK_SNDBUF    ...
#define ARES_OPT_SOCK_RCVBUF    ...
#define ARES_OPT_TIMEOUTMS      ...
#define ARES_OPT_ROTATE         ...
#define ARES_OPT_EDNSPSZ        ...
#define ARES_OPT_RESOLVCONF     ...
#define ARES_OPT_EVENT_THREAD   ...

#define ARES_NI_NOFQDN                  ...
#define ARES_NI_NUMERICHOST             ...
#define ARES_NI_NAMEREQD                ...
#define ARES_NI_NUMERICSERV             ...
#define ARES_NI_DGRAM                   ...
#define ARES_NI_TCP                     ...
#define ARES_NI_UDP                     ...
#define ARES_NI_SCTP                    ...
#define ARES_NI_DCCP                    ...
#define ARES_NI_NUMERICSCOPE            ...
#define ARES_NI_LOOKUPHOST              ...
#define ARES_NI_LOOKUPSERVICE           ...
#define ARES_NI_IDN                     ...
#define ARES_NI_IDN_ALLOW_UNASSIGNED    ...
#define ARES_NI_IDN_USE_STD3_ASCII_RULES ...

#define ARES_AI_CANONNAME               ...
#define ARES_AI_NUMERICHOST             ...
#define ARES_AI_PASSIVE                 ...
#define ARES_AI_NUMERICSERV             ...
#define ARES_AI_V4MAPPED                ...
#define ARES_AI_ALL                     ...
#define ARES_AI_ADDRCONFIG              ...
#define ARES_AI_IDN                     ...
#define ARES_AI_IDN_ALLOW_UNASSIGNED    ...
#define ARES_AI_IDN_USE_STD3_ASCII_RULES ...
#define ARES_AI_CANONIDN                ...
#define ARES_AI_MASK ...

#define ARES_LIB_INIT_ALL    ...

#define ARES_SOCKET_BAD ...

typedef enum {
  ARES_FALSE = 0,
  ARES_TRUE  = 1
} ares_bool_t;

typedef enum {
  ARES_SUCCESS = 0,

  /* Server error codes (ARES_ENODATA indicates no relevant answer) */
  ARES_ENODATA   = 1,
  ARES_EFORMERR  = 2,
  ARES_ESERVFAIL = 3,
  ARES_ENOTFOUND = 4,
  ARES_ENOTIMP   = 5,
  ARES_EREFUSED  = 6,

  /* Locally generated error codes */
  ARES_EBADQUERY    = 7,
  ARES_EBADNAME     = 8,
  ARES_EBADFAMILY   = 9,
  ARES_EBADRESP     = 10,
  ARES_ECONNREFUSED = 11,
  ARES_ETIMEOUT     = 12,
  ARES_EOF          = 13,
  ARES_EFILE        = 14,
  ARES_ENOMEM       = 15,
  ARES_EDESTRUCTION = 16,
  ARES_EBADSTR      = 17,

  /* ares_getnameinfo error codes */
  ARES_EBADFLAGS = 18,

  /* ares_getaddrinfo error codes */
  ARES_ENONAME   = 19,
  ARES_EBADHINTS = 20,

  /* Uninitialized library error code */
  ARES_ENOTINITIALIZED = 21, /* introduced in 1.7.0 */

  /* ares_library_init error codes */
  ARES_ELOADIPHLPAPI         = 22, /* introduced in 1.7.0 */
  ARES_EADDRGETNETWORKPARAMS = 23, /* introduced in 1.7.0 */

  /* More error codes */
  ARES_ECANCELLED = 24, /* introduced in 1.7.0 */

  /* More ares_getaddrinfo error codes */
  ARES_ESERVICE = 25, /* ares_getaddrinfo() was passed a text service name that
                       * is not recognized. introduced in 1.16.0 */

  ARES_ENOSERVER = 26 /* No DNS servers were configured */
} ares_status_t;

typedef void (*ares_sock_state_cb)(void *data,
                                   ares_socket_t socket_fd,
                                   int readable,
                                   int writable);

typedef void (*ares_callback_dnsrec)(void *arg,
                                     ares_status_t status,
                                     size_t timeouts,
                                     const ares_dns_record_t *dnsrec);

typedef void (*ares_host_callback)(void *arg,
                                   int status,
                                   int timeouts,
                                   struct hostent *hostent);

typedef void (*ares_nameinfo_callback)(void *arg,
                                       int status,
                                       int timeouts,
                                       char *node,
                                       char *service);

typedef int  (*ares_sock_create_callback)(ares_socket_t socket_fd,
                                          int type,
                                          void *data);

typedef void (*ares_addrinfo_callback)(void *arg,
                                   int status,
                                   int timeouts,
                                   struct ares_addrinfo *res);

struct ares_channeldata;
typedef struct ares_channeldata *ares_channel;

struct ares_server_failover_options {
  unsigned short retry_chance;
  size_t         retry_delay;
};

/*! Values for ARES_OPT_EVENT_THREAD */
typedef enum {
  /*! Default (best choice) event system */
  ARES_EVSYS_DEFAULT = 0,
  /*! Win32 IOCP/AFD_POLL event system */
  ARES_EVSYS_WIN32 = 1,
  /*! Linux epoll */
  ARES_EVSYS_EPOLL = 2,
  /*! BSD/MacOS kqueue */
  ARES_EVSYS_KQUEUE = 3,
  /*! POSIX poll() */
  ARES_EVSYS_POLL = 4,
  /*! last fallback on Unix-like systems, select() */
  ARES_EVSYS_SELECT = 5
} ares_evsys_t;

struct ares_options {
  int flags;
  int timeout; /* in seconds or milliseconds, depending on options */
  int tries;
  int ndots;
  unsigned short udp_port; /* host byte order */
  unsigned short tcp_port; /* host byte order */
  int socket_send_buffer_size;
  int socket_receive_buffer_size;
  struct in_addr *servers;
  int nservers;
  char **domains;
  int ndomains;
  char *lookups;
  ares_sock_state_cb sock_state_cb;
  void *sock_state_cb_data;
  struct apattern *sortlist;
  int nsort;
  int ednspsz;
  char *resolvconf_path;
  char *hosts_path;
  int udp_max_queries;
  int maxtimeout; /* in milliseconds */
  unsigned int qcache_max_ttl; /* Maximum TTL for query cache, 0=disabled */
  ares_evsys_t evsys;
  struct ares_server_failover_options server_failover_opts;
  ...;
};

struct ares_in6_addr {
  union {
    unsigned char _S6_u8[16];
  } _S6_un;
};

struct ares_addrttl {
  struct in_addr ipaddr;
  int            ttl;
};

struct ares_addr6ttl {
  struct ares_in6_addr ip6addr;
  int             ttl;
};

struct ares_caa_reply {
  struct ares_caa_reply  *next;
  int                     critical;
  unsigned char          *property;
  size_t                  plength;
  unsigned char          *value;
  size_t                  length;
};

struct ares_srv_reply {
  struct ares_srv_reply  *next;
  char                   *host;
  unsigned short          priority;
  unsigned short          weight;
  unsigned short          port;
};

struct ares_mx_reply {
  struct ares_mx_reply   *next;
  char                   *host;
  unsigned short          priority;
};

struct ares_txt_reply {
  struct ares_txt_reply  *next;
  unsigned char          *txt;
  size_t                  length;
};

struct ares_txt_ext {
  struct ares_txt_ext      *next;
  unsigned char            *txt;
  size_t                   length;
  unsigned char            record_start;
};

struct ares_naptr_reply {
  struct ares_naptr_reply *next;
  unsigned char           *flags;
  unsigned char           *service;
  unsigned char           *regexp;
  char                    *replacement;
  unsigned short           order;
  unsigned short           preference;
};

struct ares_soa_reply {
  char        *nsname;
  char        *hostmaster;
  unsigned int serial;
  unsigned int refresh;
  unsigned int retry;
  unsigned int expire;
  unsigned int minttl;
};
/*
 * Similar to addrinfo, but with extra ttl and missing canonname.
 */
struct ares_addrinfo_node {
  int                        ai_ttl;
  int                        ai_flags;
  int                        ai_family;
  int                        ai_socktype;
  int                        ai_protocol;
  ares_socklen_t             ai_addrlen;
  struct sockaddr           *ai_addr;
  struct ares_addrinfo_node *ai_next;
};

/*
 * alias - label of the resource record.
 * name - value (canonical name) of the resource record.
 * See RFC2181 10.1.1. CNAME terminology.
 */
struct ares_addrinfo_cname {
  int                         ttl;
  char                       *alias;
  char                       *name;
  struct ares_addrinfo_cname *next;
};

struct ares_addrinfo {
  struct ares_addrinfo_cname *cnames;
  struct ares_addrinfo_node  *nodes;
  ...;
};

struct ares_addrinfo_hints {
  int ai_flags;
  int ai_family;
  int ai_socktype;
  int ai_protocol;
};
"""

FUNCTIONS = """
int ares_library_init(int flags);

void ares_library_cleanup(void);

const char *ares_version(int *version);

int ares_init_options(ares_channel *channelptr,
                                   struct ares_options *options,
                                   int optmask);

int ares_reinit(ares_channel channel);

int ares_save_options(ares_channel channel,
                                   struct ares_options *options,
                                   int *optmask);

void ares_destroy_options(struct ares_options *options);

int ares_dup(ares_channel *dest,
                          ares_channel src);

void ares_destroy(ares_channel channel);

void ares_cancel(ares_channel channel);

void ares_set_local_ip4(ares_channel channel, unsigned int local_ip);

void ares_set_local_ip6(ares_channel channel,
                                     const unsigned char* local_ip6);

void ares_set_local_dev(ares_channel channel,
                                     const char* local_dev_name);

void ares_set_socket_callback(ares_channel channel,
                                           ares_sock_create_callback callback,
                                           void *user_data);

void ares_getaddrinfo(ares_channel channel,
                                   const char* node,
                                   const char* service,
                                   const struct ares_addrinfo_hints* hints,
                                   ares_addrinfo_callback callback,
                                   void* arg);

void ares_freeaddrinfo(struct ares_addrinfo* ai);

/* New DNS record API */
ares_status_t ares_query_dnsrec(ares_channel channel,
                                const char *name,
                                ares_dns_class_t dnsclass,
                                ares_dns_rec_type_t type,
                                ares_callback_dnsrec callback,
                                void *arg,
                                unsigned short *qid);

ares_status_t ares_search_dnsrec(ares_channel channel,
                                 const ares_dns_record_t *dnsrec,
                                 ares_callback_dnsrec callback,
                                 void *arg);

ares_status_t ares_dns_record_create(ares_dns_record_t **dnsrec,
                                     unsigned short id,
                                     unsigned short flags,
                                     ares_dns_opcode_t opcode,
                                     ares_dns_rcode_t rcode);

ares_status_t ares_dns_record_query_add(ares_dns_record_t *dnsrec,
                                        const char *name,
                                        ares_dns_rec_type_t qtype,
                                        ares_dns_class_t qclass);

void ares_dns_record_destroy(ares_dns_record_t *dnsrec);

size_t ares_dns_record_rr_cnt(const ares_dns_record_t *dnsrec,
                              ares_dns_section_t sect);

const ares_dns_rr_t *ares_dns_record_rr_get_const(const ares_dns_record_t *dnsrec,
                                                  ares_dns_section_t sect,
                                                  size_t idx);

const char *ares_dns_rr_get_name(const ares_dns_rr_t *rr);

ares_dns_rec_type_t ares_dns_rr_get_type(const ares_dns_rr_t *rr);

ares_dns_class_t ares_dns_rr_get_class(const ares_dns_rr_t *rr);

unsigned int ares_dns_rr_get_ttl(const ares_dns_rr_t *rr);

/* Record data accessors */
const struct in_addr *ares_dns_rr_get_addr(const ares_dns_rr_t *rr,
                                           ares_dns_rr_key_t key);

const struct ares_in6_addr *ares_dns_rr_get_addr6(const ares_dns_rr_t *rr,
                                                  ares_dns_rr_key_t key);

const char *ares_dns_rr_get_str(const ares_dns_rr_t *rr,
                               ares_dns_rr_key_t key);

unsigned char ares_dns_rr_get_u8(const ares_dns_rr_t *rr,
                                ares_dns_rr_key_t key);

unsigned short ares_dns_rr_get_u16(const ares_dns_rr_t *rr,
                                  ares_dns_rr_key_t key);

unsigned int ares_dns_rr_get_u32(const ares_dns_rr_t *rr,
                                ares_dns_rr_key_t key);

const unsigned char *ares_dns_rr_get_bin(const ares_dns_rr_t *rr,
                                        ares_dns_rr_key_t key,
                                        size_t *len);

size_t ares_dns_rr_get_abin_cnt(const ares_dns_rr_t *rr,
                               ares_dns_rr_key_t key);

const unsigned char *ares_dns_rr_get_abin(const ares_dns_rr_t *rr,
                                         ares_dns_rr_key_t key,
                                         size_t idx,
                                         size_t *len);

size_t ares_dns_rr_get_opt_cnt(const ares_dns_rr_t *dns_rr,
                               ares_dns_rr_key_t key);

unsigned short ares_dns_rr_get_opt(const ares_dns_rr_t *dns_rr,
                                   ares_dns_rr_key_t key,
                                   size_t idx,
                                   const unsigned char **val,
                                   size_t *val_len);

void ares_gethostbyaddr(ares_channel channel,
                                     const void *addr,
                                     int addrlen,
                                     int family,
                                     ares_host_callback callback,
                                     void *arg);

void ares_getnameinfo(ares_channel channel,
                                   const struct sockaddr *sa,
                                   ares_socklen_t salen,
                                   int flags,
                                   ares_nameinfo_callback callback,
                                   void *arg);

struct timeval *ares_timeout(ares_channel channel,
                                          struct timeval *maxtv,
                                          struct timeval *tv);

void ares_process_fd(ares_channel channel,
                                  ares_socket_t read_fd,
                                  ares_socket_t write_fd);

void ares_free_string(void *str);

void ares_free_hostent(struct hostent *host);

void ares_free_data(void *dataptr);

const char *ares_strerror(int code);

int ares_set_servers_csv(ares_channel channel, const char *servers);

char *ares_get_servers_csv(const ares_channel channel);

const char *ares_inet_ntop(int af, const void *src, char *dst,
                                        ares_socklen_t size);

int ares_inet_pton(int af, const char *src, void *dst);

ares_bool_t ares_threadsafety(void);

ares_status_t ares_queue_wait_empty(ares_channel channel, int timeout_ms);
"""

CALLBACKS = """
extern "Python" void _sock_state_cb(void *data,
                                    ares_socket_t socket_fd,
                                    int readable,
                                    int writable);

extern "Python" void _host_cb(void *arg,
                              int status,
                              int timeouts,
                              struct hostent *hostent);

extern "Python" void _nameinfo_cb(void *arg,
                                  int status,
                                  int timeouts,
                                  char *node,
                                  char *service);

extern "Python" void _query_dnsrec_cb(void *arg,
                                      ares_status_t status,
                                      size_t timeouts,
                                      const ares_dns_record_t *dnsrec);

extern "Python" void _addrinfo_cb(void *arg,
                                  int status,
                                  int timeouts,
                                  struct ares_addrinfo *res);
"""

INCLUDES = """
#ifdef _WIN32
#define WIN32_LEAN_AND_MEAN
# include <WinSock2.h>
#else
# include <sys/types.h>
# include <sys/socket.h>
# include <netdb.h> /* struct hostent */
# include <netinet/in.h> /* struct sockaddr_in/sockaddr_in6 */
#endif
#include <ares.h>
#include <ares_dns_record.h>
"""


ffi = cffi.FFI()
ffi.cdef(PLATFORM_TYPES + TYPES + FUNCTIONS + CALLBACKS)
ffi.set_source('_cares', INCLUDES)



#!/usr/bin/env python

import functools
import gc
import ipaddress
import os
import random
import socket
import string
import sys
import threading
import time
import unittest
import weakref

import pycares

FIXTURES_PATH = os.path.realpath(os.path.join(os.path.dirname(__file__), "fixtures"))


class DNSTest(unittest.TestCase):
    def setUp(self):
        self.channel = pycares.Channel(
            timeout=10.0,
            tries=1,
            servers=["8.8.8.8", "8.8.4.4"]
        )
        self.is_ci = (
            os.environ.get("APPVEYOR")
            or os.environ.get("TRAVIS")
            or os.environ.get("GITHUB_ACTION")
        )

    def tearDown(self):
        self.channel.close()
        self.channel = None

    def wait(self):
        self.channel.wait()

    def assertNoError(self, errorno):
        if errorno == pycares.errno.ARES_ETIMEOUT and self.is_ci:
            raise unittest.SkipTest("timeout")
        self.assertEqual(errorno, None)

    @unittest.skipIf(sys.platform == "win32", "skipped on Windows")
    def test_getaddrinfo(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.getaddrinfo("localhost", 80, callback=cb)
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.AddrInfoResult)
        self.assertTrue(len(self.result.nodes) > 0)
        for node in self.result.nodes:
            self.assertEqual(node.addr[1], 80)

    @unittest.skipIf(sys.platform == "win32", "skipped on Windows")
    def test_getaddrinfo2(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.getaddrinfo("localhost", "http", callback=cb)
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.AddrInfoResult)
        self.assertTrue(len(self.result.nodes) > 0)
        for node in self.result.nodes:
            self.assertEqual(node.addr[1], 80)

    @unittest.skipIf(sys.platform == "win32", "skipped on Windows")
    def test_getaddrinfo3(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.getaddrinfo("localhost", None, callback=cb)
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.AddrInfoResult)
        self.assertTrue(len(self.result.nodes) > 0)
        for node in self.result.nodes:
            self.assertEqual(node.addr[1], 0)

    @unittest.skipIf(sys.platform == "win32", "skipped on Windows")
    def test_getaddrinfo4(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.getaddrinfo("localhost", "http", family=socket.AF_INET, callback=cb)
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.AddrInfoResult)
        self.assertEqual(len(self.result.nodes), 1)
        node = self.result.nodes[0]
        self.assertEqual(node.addr[0], b"127.0.0.1")
        self.assertEqual(node.addr[1], 80)

    def test_getaddrinfo5(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.getaddrinfo("google.com", "http", callback=cb)
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.AddrInfoResult)
        self.assertTrue(len(self.result.nodes) > 0)

    @unittest.skipIf(sys.platform == "win32", "skipped on Windows")
    def test_gethostbyaddr(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.gethostbyaddr("127.0.0.1", callback=cb)
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.HostResult)

    @unittest.skipIf(sys.platform == "win32", "skipped on Windows")
    def test_gethostbyaddr6(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.gethostbyaddr("::1", callback=cb)
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.HostResult)

    @unittest.skipIf(sys.platform == "win32", "skipped on Windows")
    def test_getnameinfo(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.getnameinfo(
            ("127.0.0.1", 80),
            pycares.ARES_NI_LOOKUPHOST | pycares.ARES_NI_LOOKUPSERVICE,
            callback=cb,
        )
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.NameInfoResult)
        self.assertIn(self.result.node, ("localhost.localdomain", "localhost"))
        self.assertEqual(self.result.service, "http")

    @unittest.skipIf(sys.platform == "win32", "skipped on Windows")
    @unittest.expectedFailure  # c-ares is broken (does not return numeric service if asked) and unconditionally adds zero scope
    def test_getnameinfo_ipv6(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.getnameinfo(
            ("fd01:dec0:0:1::2020", 80, 0, 0),
            pycares.ARES_NI_NUMERICHOST | pycares.ARES_NI_NUMERICSERV,
            callback=cb,
        )
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.NameInfoResult)
        self.assertEqual(self.result.node, "fd01:dec0:0:1::2020")
        self.assertEqual(self.result.service, "80")

    @unittest.skipIf(sys.platform == "win32", "skipped on Windows")
    @unittest.expectedFailure  # c-ares is broken (does not return numeric service if asked)
    def test_getnameinfo_ipv6_ll(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.getnameinfo(
            ("fe80::5abd:fee7:4177:60c0", 80, 0, 666),
            pycares.ARES_NI_NUMERICHOST
            | pycares.ARES_NI_NUMERICSERV
            | pycares.ARES_NI_NUMERICSCOPE,
            callback=cb,
        )
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.NameInfoResult)
        self.assertEqual(self.result.node, "fe80::5abd:fee7:4177:60c0%666")
        self.assertEqual(self.result.service, "80")

    def test_query_a(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.query("google.com", pycares.QUERY_TYPE_A, callback=cb)
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.DNSResult)
        self.assertGreater(len(self.result.answer), 0)
        for record in self.result.answer:
            self.assertEqual(type(record), pycares.DNSRecord)
            self.assertEqual(type(record.data), pycares.ARecordData)
            self.assertNotEqual(record.data.addr, None)
            self.assertGreater(record.ttl, 0)  # Real TTL values now!

    def test_query_a_bad(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.query("hgf8g2od29hdohid.com", pycares.QUERY_TYPE_A, callback=cb)
        self.wait()
        self.assertEqual(self.result, None)
        self.assertEqual(self.errorno, pycares.errno.ARES_ENOTFOUND)

    def test_query_a_rotate(self):
        self.result, self.errorno = None, None
        self.errorno_count, self.count = 0, 0

        def cb(result, errorno):
            self.result, self.errorno = result, errorno
            if errorno:
                self.errorno_count += 1
            self.count += 1

        self.channel = pycares.Channel(timeout=10.0, tries=1, rotate=True)
        self.channel.query("google.com", pycares.QUERY_TYPE_A, callback=cb)
        self.channel.query("google.com", pycares.QUERY_TYPE_A, callback=cb)
        self.channel.query("google.com", pycares.QUERY_TYPE_A, callback=cb)
        self.wait()
        self.assertEqual(self.count, 3)
        self.assertEqual(self.errorno_count, 0)

    def test_query_aaaa(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.query("ipv6.google.com", pycares.QUERY_TYPE_AAAA, callback=cb)
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.DNSResult)
        self.assertGreater(len(self.result.answer), 0)
        # DNS may return CNAME records first, followed by AAAA records
        aaaa_records = [r for r in self.result.answer if type(r.data) == pycares.AAAARecordData]
        self.assertGreater(len(aaaa_records), 0, "Expected at least one AAAA record")
        for record in aaaa_records:
            self.assertNotEqual(record.data.addr, None)
            self.assertGreater(record.ttl, 0)

    def test_query_caa(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.query("wikipedia.org", pycares.QUERY_TYPE_CAA, callback=cb)
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.DNSResult)
        self.assertGreater(len(self.result.answer), 0)
        for record in self.result.answer:
            self.assertEqual(type(record.data), pycares.CAARecordData)

    def test_query_cname(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.query("www.amazon.com", pycares.QUERY_TYPE_CNAME, callback=cb)
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.DNSResult)
        self.assertGreater(len(self.result.answer), 0)
        self.assertEqual(type(self.result.answer[0].data), pycares.CNAMERecordData)

    def test_query_mx(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.query("google.com", pycares.QUERY_TYPE_MX, callback=cb)
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.DNSResult)
        self.assertGreater(len(self.result.answer), 0)
        for record in self.result.answer:
            self.assertEqual(type(record.data), pycares.MXRecordData)

    def test_query_ns(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.query("google.com", pycares.QUERY_TYPE_NS, callback=cb)
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.DNSResult)
        self.assertGreater(len(self.result.answer), 0)
        for record in self.result.answer:
            self.assertEqual(type(record.data), pycares.NSRecordData)

    def test_query_txt(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.query("google.com", pycares.QUERY_TYPE_TXT, callback=cb)
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.DNSResult)
        self.assertGreater(len(self.result.answer), 0)
        for record in self.result.answer:
            self.assertEqual(type(record.data), pycares.TXTRecordData)

    def test_query_txt_chunked(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.query("jobscoutdaily.com", pycares.QUERY_TYPE_TXT, callback=cb)
        self.wait()
        self.assertNoError(self.errorno)
        # If the chunks are aggregated, only one TXT record should be visible.
        # Three would show if they are not properly merged.
        self.assertEqual(type(self.result), pycares.DNSResult)
        self.assertEqual(len(self.result.answer), 1)
        self.assertTrue(self.result.answer[0].data.data.startswith(b"v=spf1 A MX"))

    def test_query_txt_multiple_chunked(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.query("google.com", pycares.QUERY_TYPE_TXT, callback=cb)
        self.wait()
        self.assertNoError(self.errorno)
        # > dig -t txt google.com
        # google.com.		3270	IN	TXT	"google-site-verification=TV9-DBe4R80X4v0M4U_bd_J9cpOJM0nikft0jAgjmsQ"
        # google.com.		3270	IN	TXT	"atlassian-domain-verification=5YjTmWmjI92ewqkx2oXmBaD60Td9zWon9r6eakvHX6B77zzkFQto8PQ9QsKnbf4I"
        # google.com.		3270	IN	TXT	"docusign=05958488-4752-4ef2-95eb-aa7ba8a3bd0e"
        # google.com.		3270	IN	TXT	"facebook-domain-verification=22rm551cu4k0ab0bxsw536tlds4h95"
        # google.com.		3270	IN	TXT	"google-site-verification=wD8N7i1JTNTkezJ49swvWW48f8_9xveREV4oB-0Hf5o"
        # google.com.		3270	IN	TXT	"apple-domain-verification=30afIBcvSuDV2PLX"
        # google.com.		3270	IN	TXT	"webexdomainverification.8YX6G=6e6922db-e3e6-4a36-904e-a805c28087fa"
        # google.com.		3270	IN	TXT	"MS=E4A68B9AB2BB9670BCE15412F62916164C0B20BB"
        # google.com.		3270	IN	TXT	"v=spf1 include:_spf.google.com ~all"
        # google.com.		3270	IN	TXT	"globalsign-smime-dv=CDYX+XFHUw2wml6/Gb8+59BsH31KzUr6c1l2BPvqKX8="
        # google.com.		3270	IN	TXT	"docusign=1b0a6754-49b1-4db5-8540-d2c12664b289"
        self.assertEqual(type(self.result), pycares.DNSResult)
        self.assertGreater(len(self.result.answer), 10)

    def test_query_txt_bytes1(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.query("google.com", pycares.QUERY_TYPE_TXT, callback=cb)
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.DNSResult)
        for record in self.result.answer:
            self.assertEqual(type(record.data), pycares.TXTRecordData)
            self.assertIsInstance(record.data.data, bytes)

    # The 2 tests below hit a dead end thus fail. Commenting for now as I couldn't find a live server
    # that satisfies what the tests are looking for

    # FIXME: wide.com.es is a dead end!
    @unittest.expectedFailure
    def test_query_txt_bytes2(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.query("wide.com.es", pycares.QUERY_TYPE_TXT, callback=cb)
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.DNSResult)
        for record in self.result.answer:
            self.assertEqual(type(record.data), pycares.TXTRecordData)
            self.assertIsInstance(record.data.data, bytes)

    # FIXME: "txt-non-ascii.dns-test.hmnid.ru" is a dead end!
    @unittest.expectedFailure
    def test_query_txt_multiple_chunked_with_non_ascii_content(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.query(
            "txt-non-ascii.dns-test.hmnid.ru", pycares.QUERY_TYPE_TXT, callback=cb
        )
        self.wait()
        self.assertNoError(self.errorno)
        # txt-non-ascii.dns-test.hmnid.ru.        IN      TXT     "ascii string" "some\208misc\208stuff"

        self.assertEqual(type(self.result), pycares.DNSResult)
        self.assertEqual(len(self.result.answer), 1)
        record = self.result.answer[0]
        self.assertEqual(type(record.data), pycares.TXTRecordData)
        self.assertIsInstance(record.data.data, bytes)

    def test_query_class_chaos(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.servers = ["199.7.83.42"]  # l.root-servers.net
        self.channel.query(
            "id.server", pycares.QUERY_TYPE_TXT, query_class=pycares.QUERY_CLASS_CHAOS, callback=cb
        )
        self.wait()
        self.assertNoError(self.errorno)
        # id.server.              0       CH      TXT     "aa.de-ham.l.root"

        self.assertEqual(type(self.result), pycares.DNSResult)
        self.assertEqual(len(self.result.answer), 1)
        record = self.result.answer[0]
        self.assertEqual(type(record.data), pycares.TXTRecordData)
        self.assertIsInstance(record.data.data, bytes)

    def test_query_class_invalid(self):
        with self.assertRaises(ValueError):
            self.channel.query(
                "google.com",
                pycares.QUERY_TYPE_A,
                query_class="INVALIDTYPE",
                callback=lambda *x: None,
            )
        self.wait()

    def test_query_soa(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.query("google.com", pycares.QUERY_TYPE_SOA, callback=cb)
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.DNSResult)
        self.assertGreater(len(self.result.answer), 0)
        self.assertEqual(type(self.result.answer[0].data), pycares.SOARecordData)

    def test_query_srv(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.query("_xmpp-server._tcp.jabber.org", pycares.QUERY_TYPE_SRV, callback=cb)
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.DNSResult)
        self.assertGreater(len(self.result.answer), 0)
        for record in self.result.answer:
            self.assertEqual(type(record.data), pycares.SRVRecordData)

    def test_query_naptr(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.query("sip2sip.info", pycares.QUERY_TYPE_NAPTR, callback=cb)
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.DNSResult)
        self.assertGreater(len(self.result.answer), 0)
        for record in self.result.answer:
            self.assertEqual(type(record.data), pycares.NAPTRRecordData)

    def test_query_ptr(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        ip = "172.253.122.26"
        self.channel.query(
            ipaddress.ip_address(ip).reverse_pointer, pycares.QUERY_TYPE_PTR, callback=cb
        )
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.DNSResult)
        self.assertGreater(len(self.result.answer), 0)
        self.assertEqual(type(self.result.answer[0].data), pycares.PTRRecordData)

    def test_query_ptr_ipv6(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        ip = "2001:4860:4860::8888"
        self.channel.query(
            ipaddress.ip_address(ip).reverse_pointer, pycares.QUERY_TYPE_PTR, callback=cb
        )
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.DNSResult)
        self.assertGreater(len(self.result.answer), 0)
        self.assertEqual(type(self.result.answer[0].data), pycares.PTRRecordData)

    def test_query_tlsa(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        # DANE-enabled domain with TLSA records
        self.channel.query("_25._tcp.mail.ietf.org", pycares.QUERY_TYPE_TLSA, callback=cb)
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.DNSResult)
        self.assertGreater(len(self.result.answer), 0)
        for record in self.result.answer:
            self.assertEqual(type(record.data), pycares.TLSARecordData)
            # Verify TLSA fields are present
            self.assertIsInstance(record.data.cert_usage, int)
            self.assertIsInstance(record.data.selector, int)
            self.assertIsInstance(record.data.matching_type, int)
            self.assertIsInstance(record.data.cert_association_data, bytes)

    def test_query_https(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        # Cloudflare has HTTPS records
        self.channel.query("cloudflare.com", pycares.QUERY_TYPE_HTTPS, callback=cb)
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.DNSResult)
        self.assertGreater(len(self.result.answer), 0)
        for record in self.result.answer:
            self.assertEqual(type(record.data), pycares.HTTPSRecordData)
            # Verify HTTPS fields are present
            self.assertIsInstance(record.data.priority, int)
            self.assertIsInstance(record.data.target, str)
            self.assertIsInstance(record.data.params, list)

    @unittest.skip("ANY type does not work on Mac.")
    def test_query_any(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.query("google.com", pycares.QUERY_TYPE_ANY, callback=cb)
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.DNSResult)
        self.assertGreater(len(self.result.answer), 1)

    def test_query_cancelled(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.query("google.com", pycares.QUERY_TYPE_NS, callback=cb)
        self.channel.cancel()
        self.wait()
        self.assertEqual(self.result, None)
        self.assertEqual(self.errorno, pycares.errno.ARES_ECANCELLED)

    def test_reinit(self):
        servers = self.channel.servers
        self.channel.reinit()
        self.assertEqual(servers, self.channel.servers)

    def test_query_bad_type(self):
        with self.assertRaises(ValueError):
            self.channel.query("google.com", 667, callback=lambda *x: None)
        self.wait()

    def test_query_timeout(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.servers = ["1.2.3.4"]
        self.channel.query("google.com", pycares.QUERY_TYPE_A, callback=cb)
        self.wait()
        self.assertEqual(self.result, None)
        self.assertEqual(self.errorno, pycares.errno.ARES_ETIMEOUT)

    def test_query_onion(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        # With the new API, ares_query_dnsrec may raise immediately if query can't be queued
        try:
            self.channel.query("foo.onion", pycares.QUERY_TYPE_A, callback=cb)
            self.wait()
            self.assertEqual(self.result, None)
            self.assertEqual(self.errorno, pycares.errno.ARES_ENOTFOUND)
        except pycares.AresError as e:
            # Error raised immediately - this is also valid
            self.assertEqual(e.args[0], pycares.errno.ARES_ENOTFOUND)

    def test_channel_nameservers(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel = pycares.Channel(timeout=5.0, tries=1, servers=["8.8.8.8"])
        self.channel.query("google.com", pycares.QUERY_TYPE_A, callback=cb)
        self.wait()
        self.assertNoError(self.errorno)

    def test_channel_nameservers2(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.servers = ["8.8.8.8"]
        self.channel.query("google.com", pycares.QUERY_TYPE_A, callback=cb)
        self.wait()
        self.assertNoError(self.errorno)

    def test_channel_nameservers3(self):
        servers = ["8.8.8.8", "8.8.4.4"]
        self.channel.servers = servers
        servers2 = self.channel.servers
        # CSV API includes default port :53
        self.assertEqual(servers2, ["8.8.8.8:53", "8.8.4.4:53"])

    def test_channel_local_ip(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel = pycares.Channel(
            timeout=5.0,
            tries=1,
            servers=["8.8.8.8"],
            local_ip="127.0.0.1"
        )
        # With the new API, ares_query_dnsrec may raise immediately if query can't be queued
        try:
            self.channel.query("google.com", pycares.QUERY_TYPE_A, callback=cb)
            self.wait()
            self.assertEqual(self.result, None)
            # May raise ECONNREFUSED or ETIMEDOUT depending on the platform
            self.assertIn(
                self.errorno, (pycares.errno.ARES_ECONNREFUSED, pycares.errno.ARES_ETIMEOUT)
            )
        except pycares.AresError as e:
            # Error raised immediately - this is also valid
            self.assertIn(
                e.args[0], (pycares.errno.ARES_ECONNREFUSED, pycares.errno.ARES_ETIMEOUT)
            )

    def test_channel_local_ip2(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.servers = ["8.8.8.8"]
        self.channel.set_local_ip("127.0.0.1")
        # With the new API, ares_query_dnsrec may raise immediately if query can't be queued
        try:
            self.channel.query("google.com", pycares.QUERY_TYPE_A, callback=cb)
            self.wait()
            self.assertEqual(self.result, None)
            # May raise ECONNREFUSED or ETIMEDOUT depending on the platform
            self.assertIn(
                self.errorno, (pycares.errno.ARES_ECONNREFUSED, pycares.errno.ARES_ETIMEOUT)
            )
        except pycares.AresError as e:
            # Error raised immediately - this is also valid
            self.assertIn(
                e.args[0], (pycares.errno.ARES_ECONNREFUSED, pycares.errno.ARES_ETIMEOUT)
            )
        self.assertRaises(ValueError, self.channel.set_local_ip, "an invalid ip")

    def test_channel_timeout(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        def dummy(*args):
            pass

        # Here we are explicitly not using the event thread, so lookups don't even start.
        # What we are trying to test is that cancellation works if a query is slow enough.
        self.channel = pycares.Channel(timeout=0.5, tries=1, sock_state_cb=dummy)
        self.channel.getaddrinfo("google.com", None, family=socket.AF_INET, callback=cb)
        timeout = self.channel.timeout()
        self.assertTrue(timeout > 0.0)
        self.channel.cancel()
        self.wait()
        self.assertEqual(self.result, None)
        self.assertEqual(self.errorno, pycares.errno.ARES_ECANCELLED)

    def test_import_errno(self):
        from pycares.errno import ARES_SUCCESS

        self.assertTrue(True)

    # FIXME
    @unittest.skip("The site used for this test no longer returns a non-ascii SOA.")
    def test_result_not_ascii(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.query("ayesas.com", pycares.QUERY_TYPE_SOA, callback=cb)
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.ares_query_soa_result)
        self.assertIsInstance(self.result.hostmaster, bytes)  # it's not ASCII

    def test_idna_encoding(self):
        host = "españa.icom.museum"
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        # try encoding it as utf-8
        self.channel.getaddrinfo(host.encode(), None, family=socket.AF_INET, callback=cb)
        self.wait()
        self.assertNotEqual(self.errorno, None)
        self.assertEqual(self.result, None)
        # use it as is (it's IDNA encoded internally)
        self.channel.getaddrinfo(host, None, family=socket.AF_INET, callback=cb)
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.AddrInfoResult)

    def test_idna_encoding_query_a(self):
        host = "españa.icom.museum"
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        # try encoding it as utf-8
        # With the new API, ares_query_dnsrec may raise immediately for bad names
        try:
            self.channel.query(host.encode(), pycares.QUERY_TYPE_A, callback=cb)
            self.wait()
            # ARES_EBADNAME correct for c-ares 1.24 and ARES_ENOTFOUND for 1.18
            # in 1.32.0 it was changed to ARES_ENOMEM
            if self.errorno in (pycares.errno.ARES_ENOTFOUND, pycares.errno.ARES_ENOMEM):
                self.errorno = pycares.errno.ARES_EBADNAME
            self.assertEqual(self.errorno, pycares.errno.ARES_EBADNAME)
            self.assertEqual(self.result, None)
        except pycares.AresError as e:
            # Error raised immediately - this is also valid
            error_code = e.args[0]
            if error_code in (pycares.errno.ARES_ENOTFOUND, pycares.errno.ARES_ENOMEM):
                error_code = pycares.errno.ARES_EBADNAME
            self.assertEqual(error_code, pycares.errno.ARES_EBADNAME)

        # use it as is (it's IDNA encoded internally)
        self.channel.query(host, pycares.QUERY_TYPE_A, callback=cb)
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.DNSResult)
        self.assertGreater(len(self.result.answer), 0)
        for record in self.result.answer:
            if type(record.data) == pycares.ARecordData:
                self.assertNotEqual(record.data.addr, None)
                self.assertGreater(record.ttl, 0)

    def test_idna2008_encoding(self):
        try:
            import idna
        except ImportError:
            raise unittest.SkipTest("idna module not installed")
        host = "straße.de"
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.getaddrinfo(host, None, family=socket.AF_INET, callback=cb)
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.HostResult)
        self.assertTrue("81.169.145.78" in self.result.addresses)

    @unittest.skipIf(sys.platform == "win32", "skipped on Windows")
    @unittest.skipIf(
        sys.platform == "darwin",
        "skipped on MacOS since resolver may work even if resolv.conf is broken",
    )
    def test_custom_resolvconf(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel = pycares.Channel(
            tries=1,
            timeout=2.0,
            resolvconf_path=os.path.join(FIXTURES_PATH, "badresolv.conf")
        )
        self.channel.query("google.com", pycares.QUERY_TYPE_A, callback=cb)
        self.wait()
        self.assertEqual(self.result, None)
        # TODO: some runners fail with ARES_ECONNREFUSED, which may make sense...
        # self.assertEqual(self.errorno, pycares.errno.ARES_ETIMEOUT)

    def test_errorcode_dict(self):
        for err in ("ARES_SUCCESS", "ARES_ENODATA", "ARES_ECANCELLED"):
            val = getattr(pycares.errno, err)
            self.assertEqual(pycares.errno.errorcode[val], err)

    def test_search(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel = pycares.Channel(timeout=5.0, tries=1, domains=["google.com"])
        self.channel.search("www", pycares.QUERY_TYPE_A, callback=cb)
        self.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.DNSResult)
        self.assertGreater(len(self.result.answer), 0)
        for record in self.result.answer:
            if type(record.data) == pycares.ARecordData:
                self.assertNotEqual(record.data.addr, None)
                self.assertGreater(record.ttl, 0)

    def test_lookup(self):
        self.channel = pycares.Channel(
            lookups="b",
            timeout=1,
            tries=1,
            socket_receive_buffer_size=4096,
            servers=["8.8.8.8", "8.8.4.4"],
            tcp_port=53,
            udp_port=53,
            rotate=True
        )

        def on_result(result, errorno):
            self.result, self.errorno = result, errorno

        for domain in [
            "google.com",
            "microsoft.com",
            "apple.com",
            "amazon.com",
            "baidu.com",
            "alipay.com",
            "tencent.com",
        ]:
            self.result, self.errorno = None, None
            self.channel.query(domain, pycares.QUERY_TYPE_A, callback=on_result)
            self.wait()
            self.assertNoError(self.errorno)
            self.assertTrue(self.result is not None)
            self.assertEqual(type(self.result), pycares.DNSResult)
            self.assertGreater(len(self.result.answer), 0)
            for record in self.result.answer:
                if type(record.data) == pycares.ARecordData:
                    self.assertNotEqual(record.data.addr, None)
                    self.assertGreater(record.ttl, 0)

    def test_strerror_str(self):
        for key in pycares.errno.errorcode:
            self.assertTrue(type(pycares.errno.strerror(key)), str)


class ChannelCloseTest(unittest.TestCase):
    def test_close_from_same_thread(self):
        # Test that close() works when called from the same thread
        channel = pycares.Channel()

        # Start a query
        result = []

        def cb(res, err):
            result.append((res, err))

        channel.query("google.com", pycares.QUERY_TYPE_A, callback=cb)

        # Close should work fine from same thread
        channel.close()

        # Channel should be closed, no more operations allowed
        with self.assertRaises(Exception):
            channel.query("google.com", pycares.QUERY_TYPE_A, callback=cb)

    def test_close_from_different_thread_safe(self):
        # Test that close() can be safely called from different thread
        channel = pycares.Channel()
        close_complete = threading.Event()

        def close_in_thread():
            channel.close()
            close_complete.set()

        thread = threading.Thread(target=close_in_thread)
        thread.start()
        thread.join()

        # Should complete without errors
        self.assertTrue(close_complete.is_set())
        # Channel should be destroyed
        self.assertIsNone(channel._channel)

    def test_close_idempotent(self):
        # Test that close() can be called multiple times
        channel = pycares.Channel()
        channel.close()
        channel.close()  # Should not raise

    def test_threadsafe_close(self):
        # Test that close() can be called from any thread
        channel = pycares.Channel()
        close_complete = threading.Event()

        # Close from another thread
        def close_in_thread():
            channel.close()
            close_complete.set()

        thread = threading.Thread(target=close_in_thread)
        thread.start()
        thread.join()

        self.assertTrue(close_complete.is_set())
        self.assertIsNone(channel._channel)

    def test_threadsafe_close_with_pending_queries(self):
        def dummy(*args):
            pass

        # Test close with queries in flight. Use a dummy socket state callback
        # to make sure queries don't start immediately.
        channel = pycares.Channel(sock_state_cb=dummy)
        query_completed = threading.Event()
        cancelled_count = 0

        def cb(result, error):
            nonlocal cancelled_count
            if error == pycares.errno.ARES_ECANCELLED:
                cancelled_count += 1
            if cancelled_count >= 3:  # All queries cancelled
                query_completed.set()

        # Start several queries
        channel.query("google.com", pycares.QUERY_TYPE_A, callback=cb)
        channel.query("github.com", pycares.QUERY_TYPE_A, callback=cb)
        channel.query("python.org", pycares.QUERY_TYPE_A, callback=cb)

        # Close immediately - this should cancel pending queries
        channel.close()

        # Wait for cancellation callbacks
        self.assertTrue(query_completed.wait(timeout=2.0))
        self.assertEqual(cancelled_count, 3)  # All 3 queries should be cancelled

    def test_query_after_close_raises(self):
        # Test that queries raise after close()
        channel = pycares.Channel()
        channel.close()

        def cb(result, error):
            pass

        with self.assertRaises(RuntimeError) as cm:
            channel.query("example.com", pycares.QUERY_TYPE_A, callback=cb)

        self.assertIn("destroyed", str(cm.exception))

    def test_close_from_different_thread(self):
        # Test that close works from different thread
        channel = pycares.Channel()
        close_complete = threading.Event()

        def close_in_thread():
            channel.close()
            close_complete.set()

        thread = threading.Thread(target=close_in_thread)
        thread.start()
        thread.join()

        self.assertTrue(close_complete.is_set())
        self.assertIsNone(channel._channel)

    def test_automatic_cleanup_same_thread(self):
        # Test that __del__ cleans up automatically when in same thread
        # Create a channel and weak reference to track its lifecycle
        channel = pycares.Channel()
        weak_ref = weakref.ref(channel)

        # Verify channel exists
        self.assertIsNotNone(weak_ref())

        # Delete the channel reference
        del channel

        # Force garbage collection
        gc.collect()
        gc.collect()  # Sometimes needs multiple passes

        # Channel should be gone now (cleaned up by __del__)
        self.assertIsNone(weak_ref())

    def test_automatic_cleanup_different_thread_with_shutdown_thread(self):
        # Test that __del__ now safely cleans up using shutdown thread
        # when channel is deleted from a different thread
        channel_container = []
        weak_ref_container = []

        def create_channel_in_thread():
            channel = pycares.Channel()
            weak_ref = weakref.ref(channel)
            channel_container.append(channel)
            weak_ref_container.append(weak_ref)

        # Create channel in different thread
        thread = threading.Thread(target=create_channel_in_thread)
        thread.start()
        thread.join()

        # Get the weak reference
        weak_ref = weak_ref_container[0]

        # Verify channel exists
        self.assertIsNotNone(weak_ref())

        # Delete the channel reference from main thread
        channel_container.clear()

        # Force garbage collection
        gc.collect()
        gc.collect()

        # Give the shutdown thread time to run
        time.sleep(0.1)

        # Channel should be cleaned up via the shutdown thread
        self.assertIsNone(weak_ref())

        # Note: The shutdown thread mechanism ensures safe cleanup
        # even when deleted from a different thread

    def test_no_crash_on_interpreter_shutdown(self):
        # Test that channels with pending queries don't crash during interpreter shutdown
        import subprocess

        # Path to the shutdown test script
        script_path = os.path.join(
            os.path.dirname(__file__), "shutdown_at_exit_script.py"
        )

        # Run the script in a subprocess
        result = subprocess.run(
            [sys.executable, script_path], capture_output=True, text=True
        )

        # Should exit cleanly without errors
        self.assertEqual(result.returncode, 0)
        # Should not have PythonFinalizationError in stderr
        self.assertNotIn("PythonFinalizationError", result.stderr)
        self.assertNotIn(
            "can't create new thread at interpreter shutdown", result.stderr
        )

    def test_concurrent_close_multiple_channels(self):
        # Test multiple channels being closed concurrently
        channels = []
        for _ in range(10):
            channels.append(pycares.Channel())

        close_events = []
        threads = []

        def close_channel(ch, event):
            ch.close()
            event.set()

        # Start threads to close all channels concurrently
        for ch in channels:
            event = threading.Event()
            close_events.append(event)
            thread = threading.Thread(target=close_channel, args=(ch, event))
            threads.append(thread)
            thread.start()

        # Wait for all threads to complete
        for thread in threads:
            thread.join()

        # Verify all channels were closed
        for event in close_events:
            self.assertTrue(event.is_set())

        for ch in channels:
            self.assertTrue(ch._channel is None)

    def test_rapid_channel_creation_and_close(self):
        # Test rapid creation and closing of channels
        for i in range(20):
            channel = pycares.Channel()

            # Alternate between same-thread and cross-thread closes
            if i % 2 == 0:
                channel.close()
            else:

                def close_in_thread(channel):
                    channel.close()

                thread = threading.Thread(
                    target=functools.partial(close_in_thread, channel)
                )
                thread.start()
                thread.join()

            # Verify channel is closed
            self.assertTrue(channel._channel is None)

    def test_close_with_active_queries_from_different_thread(self):
        # Test closing a channel with active queries from a different thread
        channel = pycares.Channel()
        query_started = threading.Event()
        query_cancelled = threading.Event()

        def query_cb(result, error):
            if error == pycares.errno.ARES_ECANCELLED:
                query_cancelled.set()

        # Start queries in one thread
        def start_queries():
            # Use a non-responsive server to ensure queries stay pending
            channel.servers = ["192.0.2.1"]  # TEST-NET-1, should not respond
            for i in range(5):
                channel.query(f"test{i}.example.com", pycares.QUERY_TYPE_A, callback=query_cb)
            query_started.set()

        query_thread = threading.Thread(target=start_queries)
        query_thread.start()

        # Wait for queries to start
        self.assertTrue(query_started.wait(timeout=2.0))

        # Close from main thread
        channel.close()

        # Verify channel is closed
        self.assertTrue(channel._channel is None)

        query_thread.join()

    def test_multiple_closes_from_different_threads(self):
        # Test that multiple threads can call close() safely
        channel = pycares.Channel()
        close_count = 0
        close_lock = threading.Lock()

        def close_and_count():
            channel.close()
            with close_lock:
                nonlocal close_count
                close_count += 1

        # Start multiple threads trying to close the same channel
        threads = []
        for _ in range(5):
            thread = threading.Thread(target=close_and_count)
            threads.append(thread)
            thread.start()

        # Wait for all threads
        for thread in threads:
            thread.join()

        # All threads should complete successfully
        self.assertEqual(close_count, 5)
        self.assertTrue(channel._channel is None)

    def test_stops_large_domain_name_attack(self):
        # go over 253 characters that do not include periods
        channel = pycares.Channel()
        large_domain_attack = "Ⅰ" + "".join(
            random.choices(string.ascii_letters + string.digits, k=255)
        )

        def noop(xx):
            pass

        # RuntimeError if idna is installed otherwise it should be a UnicodeError
        with self.assertRaises((RuntimeError, UnicodeError)):
            channel.query(large_domain_attack, pycares.QUERY_TYPE_A, callback=noop)


class EventThreadTest(unittest.TestCase):
    def setUp(self):
        self.channel = pycares.Channel(
            timeout=10.0, tries=1, servers=["8.8.8.8", "8.8.4.4"]
        )
        self.is_ci = (
            os.environ.get("APPVEYOR")
            or os.environ.get("TRAVIS")
            or os.environ.get("GITHUB_ACTION")
        )

    def tearDown(self):
        self.channel = None

    def assertNoError(self, errorno):
        if errorno == pycares.errno.ARES_ETIMEOUT and self.is_ci:
            raise unittest.SkipTest("timeout")
        self.assertEqual(errorno, None)

    def test_query_a(self):
        self.result, self.errorno = None, None

        def cb(result, errorno):
            self.result, self.errorno = result, errorno

        self.channel.query("google.com", pycares.QUERY_TYPE_A, callback=cb)
        self.channel.wait()
        self.assertNoError(self.errorno)
        self.assertEqual(type(self.result), pycares.DNSResult)
        self.assertGreater(len(self.result.answer), 0)
        for record in self.result.answer:
            if type(record.data) == pycares.ARecordData:
                self.assertNotEqual(record.data.addr, None)
                self.assertGreater(record.ttl, 0)


if __name__ == "__main__":
    unittest.main(verbosity=2)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import shutil
import subprocess
import sys

from setuptools import setup
from setuptools.command.build_ext import build_ext


use_system_lib = bool(int(os.environ.get('PYCARES_USE_SYSTEM_LIB', 0)))

class cares_build_ext(build_ext):
    def add_include_dir(self, dir, force=False):
        if use_system_lib and not force:
            return
        dirs = self.compiler.include_dirs
        if dir not in dirs:
            dirs.insert(0, dir)
        self.compiler.set_include_dirs(dirs)

    def build_extensions(self):
        # Use system c-ares library if requested
        if use_system_lib:
            self.compiler.add_library('cares')
            build_ext.build_extensions(self)
            return

        # Check for CMake availability
        cmake_cmd = shutil.which('cmake')
        if not cmake_cmd:
            raise RuntimeError(
                "CMake >= 3.5 is required to build pycares.\n"
                "Please install CMake from https://cmake.org/ or through your system package manager:\n"
                "  Ubuntu/Debian: apt-get install cmake\n"
                "  RHEL/CentOS/Fedora: dnf install cmake\n"
                "  macOS: brew install cmake\n"
                "  Windows: Download from https://cmake.org/download/\n"
                "  FreeBSD: pkg install cmake"
            )

        # Set up directories
        cares_dir = os.path.join('deps', 'c-ares')
        build_temp = os.path.abspath(os.path.join(self.build_temp, 'c-ares-build'))
        install_dir = os.path.abspath(os.path.join(self.build_temp, 'c-ares-install'))
        os.makedirs(build_temp, exist_ok=True)
        os.makedirs(install_dir, exist_ok=True)

        # Configure c-ares with CMake
        cmake_args = [
            '-DCARES_STATIC=ON',
            '-DCARES_SHARED=OFF',
            '-DCARES_BUILD_TOOLS=OFF',
            '-DCARES_BUILD_TESTS=OFF',
            '-DCARES_INSTALL=ON',
            '-DCARES_THREADS=ON',
            '-DCARES_STATIC_PIC=ON',  # Position independent code for static lib
            '-DCMAKE_BUILD_TYPE=Release',
            '-DCMAKE_CONFIGURATION_TYPES=Release',  # For multi-config generators (VS, Xcode)
            f'-DCMAKE_INSTALL_PREFIX={install_dir}',
        ]

        # Platform-specific configuration
        if sys.platform == 'darwin':
            # Set minimum macOS deployment target
            cmake_args.append('-DCMAKE_OSX_DEPLOYMENT_TARGET=10.12')
        elif sys.platform == 'win32':
            # Windows-specific handling
            if 'mingw' in self.compiler.compiler_type:
                cmake_args.extend(['-G', 'MinGW Makefiles'])

            # Handle static runtime if needed
            if hasattr(self, 'compiler') and hasattr(self.compiler, 'compile_options'):
                for flag in self.compiler.compile_options:
                    if '/MT' in str(flag):
                        cmake_args.append('-DCARES_MSVC_STATIC_RUNTIME=ON')
                        break

        # Run CMake configure
        print(f"Configuring c-ares with CMake in {build_temp}")
        subprocess.check_call(
            [cmake_cmd, os.path.abspath(cares_dir)] + cmake_args,
            cwd=build_temp
        )

        # Build c-ares
        print("Building c-ares...")
        build_args = ['--build', '.', '--config', 'Release']

        # Use parallel build if available
        if sys.platform != 'win32':
            try:
                import multiprocessing
                build_args.extend(['--parallel', str(multiprocessing.cpu_count())])
            except (ImportError, NotImplementedError):
                pass

        subprocess.check_call([cmake_cmd] + build_args, cwd=build_temp)

        # Install c-ares to temp directory
        print(f"Installing c-ares to {install_dir}")
        install_args = ['--install', '.', '--config', 'Release']
        subprocess.check_call([cmake_cmd] + install_args, cwd=build_temp)

        # Find the installed library
        if sys.platform == 'win32':
            # Windows libraries
            possible_paths = [
                os.path.join(install_dir, 'lib', 'cares.lib'),
                os.path.join(install_dir, 'lib', 'cares_static.lib'),
                os.path.join(install_dir, 'lib', 'libcares.a'),  # MinGW
            ]
        else:
            possible_paths = [
                os.path.join(install_dir, 'lib', 'libcares.a'),
                os.path.join(install_dir, 'lib64', 'libcares.a'),
            ]

        lib_path = None
        for path in possible_paths:
            if os.path.exists(path):
                lib_path = path
                break

        if not lib_path:
            raise RuntimeError(
                f"Could not find installed c-ares library in {install_dir}.\n"
                f"Checked: {', '.join(possible_paths)}"
            )

        print(f"Found c-ares library at: {lib_path}")

        # Set up include directories from the install directory
        # This includes all headers (original + generated like ares_build.h, ares_config.h)
        self.add_include_dir(os.path.join(install_dir, 'include'), force=True)
        # Also need internal headers from src/lib for some definitions
        self.add_include_dir(os.path.join(cares_dir, 'src', 'lib'), force=True)

        # Set up the extension to link against the static library
        self.extensions[0].extra_objects = [lib_path]

        # Add platform-specific libraries and macros
        if sys.platform == 'win32':
            self.compiler.add_library('ws2_32')
            self.compiler.add_library('advapi32')
            self.compiler.add_library('iphlpapi')
            self.compiler.define_macro('CARES_STATICLIB', 1)
        else:
            self.compiler.define_macro('HAVE_CONFIG_H', 1)
            self.compiler.define_macro('_LARGEFILE_SOURCE', 1)
            self.compiler.define_macro('_FILE_OFFSET_BITS', 64)
            self.compiler.define_macro('CARES_STATICLIB', 1)

            if sys.platform.startswith('linux'):
                self.compiler.add_library('rt')
            elif sys.platform == 'darwin':
                self.compiler.define_macro('_DARWIN_USE_64_BIT_INODE', 1)
            elif sys.platform.startswith('freebsd') or sys.platform.startswith('dragonfly'):
                self.compiler.add_library('kvm')
            elif sys.platform.startswith('sunos'):
                self.compiler.add_library('socket')
                self.compiler.add_library('nsl')
                self.compiler.add_library('kstat')

        # Build the Python extension
        build_ext.build_extensions(self)


# Most configuration has been moved to pyproject.toml
# Only build-related configuration remains here
setup(
    cmdclass={'build_ext': cares_build_ext},
    cffi_modules=['src/_cffi_src/build_cares.py:ffi'],
    ext_package='pycares',
)
